# -*- Mode: Python -*-
# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4

""" Main Codeina package """

# note: this list could be constructed at run time
INTERFACES = ['gtk', 'text', 'webkitgtk']
UI_SELECTED = 'gtk'

from codeina.extern.log import log
from codeina.myutils import env_var_is_set, has_internet 
from codeina import providers

def import_ui_string ():
    return 'codeina.ui.i'+UI_SELECTED

def run(options, patterns=None):
    """
    this function executes the different steps taken by codeina to install a
    missing plugin.
    """
    log.init('CODEINA_DEBUG')
    log.setPackageScrubList('codeina')

    if not patterns is None:
        for pattern in patterns:
            log.log('codeina', 'Pattern: %s' % pattern)

    if 'sleep_some' in options and options['sleep_some'] > 0:
        log.info('codeina', "sleeping %s seconds before starting" \
                % options['sleep_some'])
        import time
        time.sleep(options['sleep_some'])

    # Log the codeina version
    from codeina import configure
    log.log ('codeina', 'Codeina version: %s' % configure.VERSION)

    ### use text mode to update providers, so no pop ups are shown.
    ui = __import__('codeina.ui.itext', [], [], '*')  
    # check if we have some connectivity 
    no_internet = False
    if not has_internet():
        no_internet = True
        options['no_update'] = True

    products_with_update = None
    if 'no_update' in options and options['no_update']:
        log.info('codeina', "not doing updates")
    else:
        log.info('codeina', "doing updates")
        products_with_update = providers.CodeinaProvider().get_products_with_updates(id = 'fluendo')

    ### now select actual user interface
    global UI_SELECTED

    if 'ui' in options and options['ui'] in INTERFACES:
        UI_SELECTED = options['ui']
    else:
        log.warning ('codeina', "No ui selected, using default: %s" % UI_SELECTED)
    
    if not env_var_is_set('DISPLAY') and ui == 'gtk':
        UI_SELECTED = 'text'
        log.warning ('codeina', "DISPLAY environment variable is not set or empty")
        log.warning ('codeina', "switched to ui: %s" % UI_SELECTED)

    ui = None
    try:
        ui = __import__(import_ui_string (), [], [], '*')  
    except ImportError, e:
        print "Could not load interface " + UI_SELECTED
        print "ImportError:", e
        return 2

    if no_internet and not ('only_update' in options and options['only_update']):
        # we don't have internet, display a notification 
        nonet = ui.NoInternet()
        nonet()

    # should only show available updates ?
    # this mode is meant to be used at the start of a user session
    if 'only_update' in options and options['only_update']:
        if products_with_update:
            showupdates = ui.ShowUpdates()
            showupdates()
            # we have found updates, return after showing notification 1 
            log.info('codeina', "updates from Fluendo available, notification shown, only_update, exiting")
            return 0
        else:
            log.info('codeina', "no updates available and only_update, exiting")
            return 1
    log.info('codeina', "done with updating")

    if patterns:
        ret = 0
        # we have a pattern, show matches 
        for pattern in patterns:
            # check if one of the matching product is already installed
            from codeina.providers import CodeinaProvider
            providers_rank, matching_products = CodeinaProvider().get_products_for_pattern(pattern)
            already_installed = False 
            for p in matching_products:
                p.flush_cache()
                if p.is_fully_installed():
                    already_installed = True
            if already_installed: continue
            # the product is really missing, try installing it
            l = ui.ShowMatches(pattern)
            ret += l.install ()
        #return ret
    else:
        # no pattern given, and listview not asked, show fluendo shop 
        from codeina.myutils import build_matches_url
        url = build_matches_url()
        shop = ui.FluendoShop(codeina_url = url)
        ret = shop()

    log.info('codeina', "all done, exiting")
    return 0
