# -*- Mode: Python -*-
# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# Codeina, Codec Installation Application
# Copyright (C) 2007 Fluendo S.A.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

"""
Utilities for getting the location of various directories.
Enables identical use for installed and uninstalled versions.
"""

import os.path

LIBDIR = '/usr/lib64'
DATADIR = '/usr/share'
LOCALEDIR = os.path.join(DATADIR, 'locale')
PKGDATADIR = '/usr/share/codeina'
ICONSDIR = PKGDATADIR + '/icons'
VERSION = '0.10.7'
APPNAME = 'codeina'
PYGTK_REQ = '2.8.0'
PYGST_REQ = '0.10.0'
GST_REQ = '0.10.15'
SYS_PROVIDERS = '/etc/codeina/providers'
RESTRICTED_PRODUCTS = '/etc/codeina'
import xdg.BaseDirectory
USR_CODEINA   = xdg.BaseDirectory.save_data_path('codeina')
USR_PROVIDERS = xdg.BaseDirectory.save_data_path('codeina/providers')
USR_DOWNLOAD  = xdg.BaseDirectory.save_data_path('codeina/downloads')
CODEINA_SHOP_URL = 'https://www.fluendo.com/codeina/'
CODEINA_PROVIDERS_PREFIX = CODEINA_SHOP_URL + 'providers/'
PROVIDER_DEFAULT_ICON = 'media-optical.png'

def restricted_products_xml ():
    filedir = os.path.dirname(os.path.abspath(__file__))
    if os.path.exists(os.path.join(filedir, '.svn')):
        return os.path.join(filedir, '..', 'data', 'restricted-products.xml')
    else:
        return os.path.join(RESTRICTED_PRODUCTS, 'restricted-products.xml')

def make_xml_path(filename):
    filedir = os.path.dirname(os.path.abspath(__file__))
    if os.path.exists(os.path.join(filedir, '.svn')):
        return os.path.join(filedir, '..', 'providers', filename)
    else:
        return os.path.join(PKGDATADIR, 'xml', filename)

def make_icon_path(filename):
    filedir = os.path.dirname(os.path.abspath(__file__))
    if os.path.exists(os.path.join(filedir, '.svn')):
        icondir_root = os.path.join(filedir, '..', 'data')
    else:
        icondir_root = PKGDATADIR
    return os.path.join(icondir_root, 'icons', filename)

def get_codeina_icon (size):
    filename = make_icon_path ('codeina.png')
    import gtk
    pixbuf = gtk.gdk.pixbuf_new_from_file (filename)
    w = float (pixbuf.get_width ())
    h = float (pixbuf.get_height ())
    if w > h:
        h = ( h / w ) * size
        w = size
    else:
        w = ( w / h ) * size
        h = size
    return pixbuf.scale_simple (int (w), int (h), gtk.gdk.INTERP_BILINEAR)

def make_logo_path (filename):
    filedir = os.path.dirname(os.path.abspath(__file__))
    if os.path.exists(os.path.join(filedir, '.svn')):
        icondir_root = os.path.join(filedir, '..', 'data')
    else:
        icondir_root = PKGDATADIR
    return os.path.join(icondir_root, 'logo', filename)
