# vi:si:et:sw=4:sts=4:ts=4

# first of all, select reactor
from twisted.internet import gtk2reactor
try:
    # We put it in so that it doesn't try to install it
    # multiple times.
    gtk2reactor.install ()
except:
    pass
from twisted.internet import reactor

import pygtk; pygtk.require('2.0')
import gtk

from codeina.extern.log import log

from dialog import *
from notification import CodeinaNotification as Notification
from notification import CodeinaUpgradeNotification as UpgradeNotification
from fluendo_codec_buyer import FluendoGTK as FluendoShop
from codeina import ui
from gettext import gettext as _

from codeina import httpdownload
from codeina.ui.iwebkitgtk.downloadprogress import DownloadProgress
httpdownload.HttpDownload.download_object = DownloadProgress()

# Width and hight of the provider's logos
IMG_SIZE = 60

class NoCodec (CodeinaSimpleDialog, ui.NoCodec):
    pass

class NoProviders (CodeinaWarningDialog, ui.NoProviders):
    pass

class NoProducts(CodeinaWarningDialog, ui.NoProducts):
    pass

class NoInternet(Notification, ui.NoInternet):
    pass

class PackagesInstalled (CodeinaSimpleDialog, ui.PackagesInstalled):

    def __init__(self, pkgs):
        ui.PackagesInstalled.__init__ (self, pkgs)
        CodeinaSimpleDialog.__init__ (self)

class ShowUpdates(UpgradeNotification, ui.ShowUpdates):
    pass

    def create_msg (self):
        from codeina import providers 
        return '\n'.join([self.msg] + [' - ' + product.name for product in \
            providers.CodeinaProvider ().get_products_with_updates('fluendo')])

class LegalProductBox (gtk.HBox):
    def __init__ (self, product):
        gtk.HBox.__init__ (self)
        # add some padding
        p1 = gtk.Label('')
        p1.set_padding (40,0)
        self.pack_start (p1, False, False)
        # add the legal logo
        from codeina import configure
        if product.is_legal ():
            pixbuf = gtk.gdk.pixbuf_new_from_file (configure.make_icon_path('legal.png'))
        else:
            pixbuf = gtk.gdk.pixbuf_new_from_file (configure.make_icon_path('restricted.png'))
        pixbuf = pixbuf.scale_simple (25, 25, gtk.gdk.INTERP_BILINEAR)
        image = gtk.Image ()
        image.set_from_pixbuf (pixbuf)
        image.set_padding (5, 2)
        self.pack_start (image, False, False)
        # add the product name
        self.pack_start (gtk.Label (product.name), False, False)
        self.show_all ()

class LegalProviderBox (gtk.VBox):
    def __init__ (self, provider):
        def UpperFirstLetter (str):
            return str.upper ()[0] + str[1:]
        gtk.VBox.__init__ (self)
        self.pack_start (ReasonLabel (_("Install package from <b>%s</b>\n\tProvides:") % UpperFirstLetter (provider.id)), False, False)
        for prod in provider.products:
            self.pack_start (LegalProductBox (prod), False, False)
        self.show_all ()

class LegalLegendBox (gtk.VBox):

    def get_image (self, name):
        from codeina import configure
        pixbuf = gtk.gdk.pixbuf_new_from_file (configure.make_icon_path(name))
        pixbuf = pixbuf.scale_simple (40, 40, gtk.gdk.INTERP_BILINEAR)
        image = gtk.Image ()
        image.set_from_pixbuf (pixbuf)
        image.set_padding (4, 4)
        return image

    def get_label (self, text):
        label = gtk.Label ()
        label.set_markup ("<small>"+text+"</small>")
        return label

    def add_icon (self, filename, info):
        v = gtk.HBox ()
        v.pack_start (self.get_label (' '*13), False, False)
        v.pack_start (self.get_image (filename), False, False)
        v.pack_start (self.get_label (' '*7), False, False)
        v.pack_start (self.get_label (info), False, False)
        self.pack_start (v)

    def __init__ (self, reason1, reason2):
        gtk.VBox.__init__ (self)
        self.add_icon ('legal.png', reason1)
        self.add_icon ('restricted.png', reason2)
        self.show_all ()

class ReasonLabel (gtk.VBox):
    def add_line (self, line):
        hb = gtk.HBox ()
        label = gtk.Label ()
        label.set_markup(line)
        hb.pack_start (label, False, False)
        label = gtk.Label ('')
        hb.pack_start (label, True, True)
        self.pack_start (hb, False, False)

    def __init__ (self, reason):
        gtk.VBox.__init__ (self)
        for line in reason.split('\n'):
            self.add_line (line)
        self.set_border_width (5)

class ReasonLabelClickable (gtk.EventBox):

    def __init__ (self, reason, cb, data):
        gtk.EventBox.__init__ (self)
        self.add (ReasonLabel (reason))
        self.set_events(gtk.gdk.BUTTON_PRESS_MASK)
        self.connect("button_press_event", cb, data)

def string_add_wrap (string, max):
    new_string = ''
    new_line = ''
    for word in string.split(' '):
        if len (new_line + word) >= max:
            new_string += new_line + '\n'
            new_line = ''
        new_line += word + ' '
    return new_string + new_line

class ShowMatches(gtk.Dialog, ui.ShowMatches):

    def __init__ (self, pattern):

        from codeina.providers import CodeinaProvider

        # wrap the reason's
        self.reason[0] = string_add_wrap (self.reason[0], 75)
        self.reason[1] = string_add_wrap (self.reason[1], 70)
        self.reason[2] = string_add_wrap (self.reason[2], 70)

        self.pattern = pattern
        self.providers = CodeinaProvider().get_providers_for_pattern (pattern)

        gstr, vers, appl, name, caps = pattern.split('|')
        self.msg = "<b>"
        self.msg += _("The application \"%s\" requires the following plugin:") % appl
        self.msg += "</b>\n"
        self.msg += "  - " + name
        gtk.Dialog.__init__ (self, None, None, 0, None)
        self.set_title (self.dialog_title)
        # Set the icon of this dialog
        from codeina import configure
        self.set_icon_from_file (configure.make_icon_path('codeina.png'))

        stock = gtk.image_new_from_stock (gtk.STOCK_DIALOG_INFO,
                                          gtk.ICON_SIZE_DIALOG)
        lbl = gtk.Label ()
        lbl.set_text ("<big>"+self.msg+"</big>")
        lbl.set_use_markup (True)

        hbox = gtk.HBox ()
        hbox.set_border_width (8)
        hbox.set_spacing (8)

        hbox.pack_start (stock, False, False)
        hbox.pack_start (lbl, False, False)
        self.vbox.pack_start (hbox, False, False)

        def show_extra_dialog (data, widget, self):
            from dialog import CodeinaSimpleDialog
            CodeinaSimpleDialog (msg=self.reason[4]) ()

        # Add the legal explanation
        v = gtk.VBox()
        v.pack_start (ReasonLabel (self.reason[0]))
        v.pack_start (gtk.Label ())
        v.pack_start (LegalLegendBox (self.reason[1], self.reason[2]), False, False)
        v.pack_start (gtk.Label ())
        v.pack_start (ReasonLabelClickable ("<span color=\"blue\" underline=\"single\">"+self.reason[3]+"</span>", show_extra_dialog, self))
        v.pack_start (gtk.Label ())
        v.pack_start (gtk.HSeparator ())
        self.vbox.pack_start (v, False, False)
        self.vbox.pack_start (gtk.Label ())

        v = gtk.VBox()
        first_button = None
        self.selected_provider = -1

        def callback(widget, data=-1):
            self.selected_provider = data

        # Check that the Fluendo provider is the first one
        from codeina.providers import set_fluendo_provider_first
        self.providers = set_fluendo_provider_first (self.providers)

        for provider in self.providers:
            provider.check_legal ()
            if first_button == None:
                provider_radio = gtk.RadioButton(label = None, group = None)
            else:
                provider_radio = gtk.RadioButton(label = None, group = first_button)
            provider_radio.connect("toggled", callback, provider)

            if first_button == None:
                provider_radio.set_active(True)
                self.selected_provider = provider
                first_button = provider_radio

            radioHBox = gtk.HBox ()
            img = gtk.Image()
            img.set_from_pixbuf (provider.get_logo_pixbuf (IMG_SIZE))
            radioHBox.pack_start (img, False, False)
            radioHBox.pack_start (LegalProviderBox (provider))
            provider_radio.add (radioHBox)
            v.pack_start(provider_radio, False)

        v.show()
        self.vbox.pack_start(v, False, False)
        
        button = gtk.Button(stock=gtk.STOCK_CANCEL)
        self.add_action_widget(button, gtk.RESPONSE_CANCEL)

        button = gtk.Button(_("_Install"))
        image = gtk.Image ()
        image.set_from_stock(gtk.STOCK_OK, gtk.ICON_SIZE_BUTTON)
        button.set_image(image)
        button.set_flags (gtk.CAN_DEFAULT)
        button.grab_focus()
        self.add_action_widget(button, gtk.RESPONSE_YES)
        self.set_default_response(gtk.RESPONSE_YES)

    def install (self):
        # check if there are some products for this request
        if not self.providers:
            # no products to install, exits silently, returning 1
            print NoProducts.msg
            return 1

        # products found show selection
        self.vbox.show_all ()
        response = self.run ()
        if not response == gtk.RESPONSE_YES:
            log.log ('ui', 'User cancelled installation');
            return 1
        self.hide ()

        # go to the fluendo shop if we choose this provider
        if self.selected_provider.id == 'fluendo':
            import codeina.ui.iwebkitgtk as ui
            from codeina.myutils import build_matches_url
            from codeina import web
            from codeina.web import website_exists

            url = build_matches_url(self.pattern)
            if web.website_exists (url):
                shop = ui.FluendoShop(codeina_url = url)
                shop()
            else:
                # FIXME warning about site that does not exists
                from dialog import CodeinaWarningDialog
                CodeinaWarningDialog (msg=_('Failed to connect to Fluendo shop')) ()
                return 1
            return 0

        # import the correct installer factory
        from codeina import check
        distro_name = check.get_distro_name()
        if distro_name in ['ubuntu', 'debian']:
            import codeina.installer.synaptic_installer as installer_factory
            log.info ('codeina', 'using the synaptic installer')
        elif distro_name in ['mandrivalinux', 'mandriva']:
            import codeina.installer.urpmi_installer as installer_factory
            log.info ('codeina', 'using the urpmi installer')
        else:
            log.warning ('codeina', 'can not find the correct action for this provider (%s)' % self.selected_provider.id)
            return 0
        installer = installer_factory.Installer ()

        def done_cb (ret):
            gtk.main_quit ()
        # Run the installation of the packages in a different thread
        from codeina.installer import InstallerThread
        installer_thread = InstallerThread (installer, self.selected_provider, done_cb)
        installer_thread.start ()
        log.log ('codeina', 'installer is buzy destroy this dialog')
        self.destroy ()
        gtk.main ()
        # installation done
        from codeina.ui.iwebkitgtk import PackagesInstalled
        PackagesInstalled (self.selected_provider.products) ()
        log.debug ('codeina', 'all products installed')
        return 0
