# -*- Mode: Python -*-
# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4

import gtk
from codeina.extern.log import log
from codeina.ui.downloadprogress import DownloadProgress as DownloadProgressBase
from window import CodeinaWindow

class DownloadProgress(CodeinaWindow, DownloadProgressBase):

    """ Show progress of download.
    """
    from codeina.httpdownload import FluendoPackageDownloader
    downloader = FluendoPackageDownloader ()

    def __init__ (self):
        CodeinaWindow.__init__ (self)

        self.progress_bar = gtk.ProgressBar ()
        self.progress_bar.set_ellipsize (True)

        self.cancel_button = gtk.Button (stock=gtk.STOCK_CANCEL)

        self.set_size_request (480, 75)

        self.vbox.set_spacing (7)
        self.vbox.pack_start (self.progress_bar)
        self.vbox.pack_start (self.cancel_button)
        self.progress_bar.show ()
        self.vbox.show_all ()

        self.error = None
    
    def _receive_part (self, outfile, dl, total):
        try:
            fraction = dl / total
        except:
            fraction = 0
        log.log ('download', 'receiving part, fraction: %d' % fraction)
        self.progress_bar.set_fraction (fraction)

    def _finalized_download (self, obj):
        self.hide ()
        self.downloader.stop ()
        log.log ('download', 'finalizing download')
        gtk.main_quit ()

    def _message_received (self, obj, message):
        log.log ('download', 'message received %s' % message)
        obj.disable ()
        self.hide ()
        gtk.main_quit ()
        self.error = (obj.status, message)

    def _wait_for_download (self, url):
        self.progress_bar.set_text ("Downloading %s..." % (url))
        log.log ('download', "Downloading %s..." % (url))
        self.progress_bar.set_fraction (0)
        self.show_all ()
        self.error = None
        if not self.downloader.running:
            self.downloader.start ()
        else:
            gtk.main ()
        if self.error:
            raise Exception (self.error)
        return True

