# -*- Mode: Python -*-
# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4

from codeina import ui
import gobject
import gtk
import webkit
from codeina.configure import USR_CODEINA

from codeina.httpdownload import HttpDownload
from codeina.configure import USR_DOWNLOAD, CODEINA_SHOP_URL
from codeina.extern.log import log

import sha
import re

from exceptions import NotImplementedError
import codeina.installer.user_world
import codeina.installer.dpkg_installer
import codeina.installer.urpmi_installer
from dialog import *
from window import CodeinaWindow
from gettext import gettext as _

# This is the regular expression we will be looking for
# in each links, that tell us how and where to download the
# tarballs, for installation purposes.
_re_download_url = re.compile ('url=(?P<url>.*?)\|sha1sum=(?P<sha1sum>.*?)\|(?P<othervars>.*)')
_re_vars = re.compile (r'(?P<name>[^=\|]+)=(?P<value>[^\|]*)\|')

def _string_to_dict (str):
    """ Convert a string formatted like the following to a dictionary :
    somevar=toto|someothervar=toto2|
    @param str: The string
    @type str: string
    """
    dict = {}
    for match in _re_vars.finditer (str):
        name = match.group ("name")
        value = match.group ("value")
        dict[name] = value
    return dict


class FluendoInstallProgress (CodeinaWindow):

    def __init__ (self, title):
        CodeinaWindow.__init__ (self, title)
        
        self.progress_bar = gtk.ProgressBar ()
        self.progress_bar.set_ellipsize (True)

        self.set_size_request (480, 40)

        self.vbox.set_spacing (7)
        self.vbox.pack_start (self.progress_bar)
        self.progress_bar.show ()
        self.vbox.show_all ()

    def set_advancement (self, fraction, text):
        self.progress_bar.set_text (text)
        self.progress_bar.set_fraction (fraction)

    def __close (self, widget):
        pass

class FluendoGTK (CodeinaWindow, ui.FluendoShop):
    """
    Fluendo Codec Buyer. Loads the website, and sniffs all the URL clicked
    to recognize a particular "Install" link, to download and install archives.
    """

    def __install_package (self, path_package, data):
        """ Determine what type of package we are dealing with, and install it with
        the correct method.

        FIXME: It would be better to guess the format according to magic numbers
        instead of file extensions as it is done now.
        """
        log.info ('codeina', 'trying to install package from path %s' % path_package)
        if path_package.endswith (".tar.bz2") or path_package.endswith (".tar.gz"):
            i = codeina.installer.user_world.Installer ()
            errors = i.install (path_package, data)
        elif path_package.endswith (".rpm"):
            i = codeina.installer.urpmi_installer.Installer ()
            errors = i.install (path_package, data)
        elif path_package.endswith (".deb"):
            i = codeina.installer.dpkg_installer.Installer ()
            errors = i.install (path_package, data)
        else:
            raise "Unknown package format"


    def _on_resource_request_starting(self, view, frame, resource, request, response):
        uri = request.get_uri()
        if not _re_download_url.search (uri):
            # This was not the Install button
            return True # Load the page normally

        import os

        downloader = HttpDownload ()

        # Disable the main window.
        self.set_sensitive (False)

        target_directory = USR_DOWNLOAD
        successful_packages = {}
        failed_packages = {}

        errors = ""
        successes = ""

        for match in _re_download_url.finditer (uri):
            # We extract the information about the package.
            url = match.group ("url")
            localfile = os.path.basename (url)
            sha1sum = match.group ("sha1sum")
            link_vars = _string_to_dict (match.group ("othervars"))
            log.debug ('codeina', 'other vars received: %r' % link_vars)
            filename = os.path.basename (url)
            # check if this file is already in our installed products
            from codeina.productfile import productFile
            if not productFile ().product_quick_add ( filename):
                result = CodeinaMessageDialog ( _("<i>%s</i> is already installed.\nRe-install this product ?") % 
                            localfile, gtk.BUTTONS_YES_NO, gtk.MESSAGE_INFO) ()
                if not result == gtk.RESPONSE_YES:
                    log.debug ('codeina', 'we do not want to reinstall this product')
                    continue
                else:
                    log.debug ('codeina', 'reinstalling this product')
            # attempt to get the session cookie from the user profile file
            try:
                from datetime import datetime
                from urllib2 import Request
                cookiesfile = open(os.path.join(USR_CODEINA, "mozembed", "cookies.txt")).readlines()
                hostname = Request(CODEINA_SHOP_URL).get_host()
                re_cookie_session = re.compile( hostname + \
                  '\t.*/.*\t(?P<cookie_expire>[0-9].*?)\tsessionid\t(?P<sessionid>.*?)$' )
                cookie_expire, sessionid = None, None
                for line in cookiesfile:
                    match = re_cookie_session.match(line.strip())
                    if not match is None:
                        sessionid = match.group('sessionid')
                        cookie_expire = match.group('cookie_expire')
                        break
                if cookie_expire is None:
                    raise ValueError('could not extract expire date')
                if sessionid is None:
                    raise ValueError('could not extract session id')
                expire_date = datetime.fromtimestamp(float(cookie_expire))
                try:
                    assert datetime.now() < expire_date
                except AssertionError:
                    return False
                expire_string = expire_date.strftime("%a, %d-%b-%Y %H:%M:%S GMT")
                cookies = {'sessionid': sessionid, 
                           'expires': expire_string,
                           'Max-Age': '1209600',
                           'Path': '/'}
            except Exception, e:
                log.debug('codeina', 'exception trying to get cookie: %s' % str(e))
                cookies = None

            try:
                downloader.run (url, target_directory, localfile, sha1sum, 
                    cookies=cookies)

                f = file (os.path.join (target_directory, filename), 'rb')
                r = f.read()
                log.debug('codeina', 'file length: %s' % len(r))
                digest = sha.new (r).hexdigest ()
                if digest == sha1sum:
                    link_vars.update({'sha1sum': digest})
                    successful_packages[filename] = link_vars
                else:
                    failed_packages[filename] = _("Incorrect sha1sum. Downloaded file is probably corrupted.")
                    try:
                        log.debug('codeina', 'deleting %s, invalid md5sum' % filename)
                        os.remove( os.path.join (target_directory, filename) )
                    except:
                        pass
                
            except Exception, e:
                log.debug('codeina', 'caught an exception %s' % str(e))
                failed_packages[filename] = 'caught an exception %s' % str(e)
        
        if failed_packages != {}:
            for k, v in failed_packages.iteritems ():
                errors += _("Failed to install package %s (%s)\n") % (k, v)
        progress = None
        if successful_packages != {}:
            progress = FluendoInstallProgress (_("Installation Progress"))
            progress.show_all ()
            total = len (successful_packages)
            i = 1.
            for (pkg, data) in successful_packages.items ():
                progress.set_advancement (i / total, _("Installing %s") % (pkg))
                try:
                    self.__install_package (os.path.join (target_directory, pkg), data)
                    successes += _("Package %s installed successfully\n") % (pkg)
                except Exception, e:
                    errors += _("Failed to install package %s: %s\n") % (pkg, str(e))
                i += 1.

        if errors != "":
            CodeinaWarningDialog (_("Found some problems during installation:\n\n%s\n%s") % (errors, successes)) ()
            if progress:
                progress.destroy ()
            # force exiting with 1 if any error happened
            os._exit(1)
        else:
            if not successes:
                CodeinaSimpleDialog (_("No new products installed")) ()
            else:
                CodeinaSimpleDialog (_("Install and download complete.\n\n %s") % (successes)) ()
            if progress:
                progress.destroy ()
            gtk.main_quit ()

        request.set_uri('about:blank')
        return False # We are not loading anyone !



    def __init__ (self, codeina_url = CODEINA_SHOP_URL):
        CodeinaWindow.__init__ (self, self.dialog_title)

        self.connect ("destroy", lambda widget: gtk.main_quit ())

        # Webkit
        self.browser = webkit.WebView()
        self.browser.open(codeina_url)
        self.browser.connect(
                'resource-request-starting',
                self._on_resource_request_starting
                )

        self.scrollable = gtk.ScrolledWindow()
        self.scrollable.props.hscrollbar_policy = gtk.POLICY_NEVER
        self.scrollable.props.vscrollbar_policy = gtk.POLICY_NEVER
        self.scrollable.add(self.browser)
        self.vbox.add(self.scrollable)
        self.show_all()

    def __call__(self):
        self.show()
        return gtk.main()
